@@ -1,7 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -25,7 +24,7 @@ from polyglot.queue import Queue, Empty
 
 def image_extensions():
     if not hasattr(image_extensions, 'ans'):
-        image_extensions.ans = [as_unicode_polyglot(x) for x in QImageReader.supportedImageFormats()]
+        image_extensions.ans = [x.data().decode('utf-8') for x in QImageReader.supportedImageFormats()]
     return image_extensions.ans
 
 
