@@ -133,14 +133,22 @@ static inline QByteArray detectDesktopEnvironment()
     return QByteArrayLiteral("UNKNOWN");
 }
 
+static inline bool
+is_color_dark(const QColor &col) {
+	int r, g, b;
+	col.getRgb(&r, &g, &b);
+	return r < 115 && g < 155 && b < 115;
+}
+
 class CalibreStyle: public QProxyStyle {
     private:
         QHash<int, QString> icon_map;
         QByteArray desktop_environment;
         QDialogButtonBox::ButtonLayout button_layout;
+        int transient_scroller;
 
     public:
-        CalibreStyle(QStyle *base, QHash<int, QString> icmap) : QProxyStyle(base), icon_map(icmap) {
+        CalibreStyle(QStyle *base, QHash<int, QString> icmap, int transient_scroller) : QProxyStyle(base), icon_map(icmap), transient_scroller(transient_scroller) {
             setObjectName(QString("calibre"));
             desktop_environment = detectDesktopEnvironment();
             button_layout = static_cast<QDialogButtonBox::ButtonLayout>(QProxyStyle::styleHint(SH_DialogButtonLayout));
@@ -163,6 +171,8 @@ class CalibreStyle: public QProxyStyle {
                     return button_layout;
                 case SH_FormLayoutFieldGrowthPolicy:
                     return QFormLayout::FieldsStayAtSizeHint;  // Do not have fields expand to fill all available space in QFormLayout
+                case SH_ScrollBar_Transient:
+                    return transient_scroller;
 				default:
                     break;
             }
@@ -187,7 +197,7 @@ class CalibreStyle: public QProxyStyle {
         void drawComplexControl(ComplexControl control, const QStyleOptionComplex * option, QPainter * painter, const QWidget * widget = 0) const {
             const QStyleOptionToolButton *toolbutton = NULL;
             switch (control) {
-                case CC_ToolButton:
+                case CC_ToolButton:  // {{{
                     // We do not want an arrow if the toolbutton has an instant popup
                     toolbutton = qstyleoption_cast<const QStyleOptionToolButton *>(option);
                     if (toolbutton && (toolbutton->features & QStyleOptionToolButton::HasMenu) && !(toolbutton->features & QStyleOptionToolButton::PopupDelay)) {
@@ -195,7 +205,7 @@ class CalibreStyle: public QProxyStyle {
                         opt.features = toolbutton->features & ~QStyleOptionToolButton::HasMenu;
                         return QProxyStyle::drawComplexControl(control, &opt, painter, widget);
                     }
-                    break;
+                    break; /// }}}
                 default:
                     break;
             }
@@ -205,18 +215,19 @@ class CalibreStyle: public QProxyStyle {
         void drawPrimitive(PrimitiveElement element, const QStyleOption * option, QPainter * painter, const QWidget * widget = 0) const {
             const QStyleOptionViewItem *vopt = NULL;
             switch (element) {
-				case PE_FrameTabBarBase:
+				case PE_FrameTabBarBase: // {{{
 					// dont draw line below tabs in dark mode as it looks bad
 					if (const QStyleOptionTabBarBase *tbb = qstyleoption_cast<const QStyleOptionTabBarBase *>(option)) {
 						if (tbb->shape == QTabBar::RoundedNorth) {
 							QColor bg = option->palette.color(QPalette::Window);
-							if (bg.valueF() < 0.45) return;
+							if (is_color_dark(bg)) return;
 						}
 					}
-					break;
-				case PE_IndicatorCheckBox:
+					break; // }}}
+
+				case PE_IndicatorCheckBox: // {{{
 					// Fix color used to draw checkbox outline in dark mode
-					if (option->palette.color(QPalette::Window).valueF() < 0.45) {
+					if (is_color_dark(option->palette.color(QPalette::Window))) {
 						baseStyle()->drawPrimitive(element, option, painter, widget);
 						painter->save();
 						painter->translate(0.5, 0.5);
@@ -230,8 +241,9 @@ class CalibreStyle: public QProxyStyle {
 						painter->restore();
 						return;
 					}
-					break;
-                case PE_PanelItemViewItem:
+					break; // }}}
+
+                case PE_PanelItemViewItem:  // {{{
                     // Highlight the current, selected item with a different background in an item view if the highlight current item property is set
                     if (option->state & QStyle::State_HasFocus && (vopt = qstyleoption_cast<const QStyleOptionViewItem *>(option)) && widget && widget->property("highlight_current_item").toBool()) {
                         QColor color = vopt->palette.color(QPalette::Normal, QPalette::Highlight);
@@ -240,15 +252,16 @@ class CalibreStyle: public QProxyStyle {
                         opt.palette.setColor(QPalette::Highlight, color);
                         return QProxyStyle::drawPrimitive(element, &opt, painter, widget);
                     }
-                    break;
-				case PE_IndicatorToolBarSeparator:
+                    break; // }}}
+
+				case PE_IndicatorToolBarSeparator:  // {{{
 					// Make toolbar separators stand out a bit more in dark themes
 					{
 						QRect rect = option->rect;
 						const int margin = 6;
 						QColor bg = option->palette.color(QPalette::Window);
 						QColor first, second;
-						if (bg.valueF() < 0.45) {
+						if (is_color_dark(bg)) {
 							first = bg.darker(115);
 							second = bg.lighter(115);
 						} else {
@@ -281,7 +294,8 @@ class CalibreStyle: public QProxyStyle {
 									rect.topRight().y() + offset + 1);
 						}
 					}
-					return;
+					return; // }}}
+
                 default:
                     break;
             }
@@ -290,7 +304,7 @@ class CalibreStyle: public QProxyStyle {
 
 		void drawControl(ControlElement element, const QStyleOption *option, QPainter *painter, const QWidget *widget) const {
 			switch(element) {
-				case CE_MenuItem:
+				case CE_MenuItem:  // {{{
 					// Draw menu separators that work in both light and dark modes
 					if (const QStyleOptionMenuItem *menuItem = qstyleoption_cast<const QStyleOptionMenuItem *>(option)) {
 						if (menuItem->menuItemType == QStyleOptionMenuItem::Separator) {
@@ -304,7 +318,7 @@ class CalibreStyle: public QProxyStyle {
 										QPalette::Text);
 								w = menuItem->fontMetrics.horizontalAdvance(menuItem->text) + margin;
 							}
-							if (menuItem->palette.color(QPalette::Window).valueF() < 0.45) painter->setPen(Qt::gray);
+							if (is_color_dark(menuItem->palette.color(QPalette::Window))) painter->setPen(Qt::gray);
 							else painter->setPen(QColor(0, 0, 0, 60).lighter(106));
 							bool reverse = menuItem->direction == Qt::RightToLeft;
 							painter->drawLine(menuItem->rect.left() + margin + (reverse ? 0 : w), menuItem->rect.center().y(),
@@ -313,16 +327,17 @@ class CalibreStyle: public QProxyStyle {
 							return;
 						}
 					}
-					break;
+					break; // }}}
+
 				default: break;
 			}
             QProxyStyle::drawControl(element, option, painter, widget);
 		}
 };
 
-int load_style(QHash<int,QString> icon_map) {
+int load_style(QHash<int,QString> icon_map, int transient_scroller) {
     QStyle *base_style = QStyleFactory::create(QString("Fusion"));
-    QApplication::setStyle(new CalibreStyle(base_style, icon_map));
+    QApplication::setStyle(new CalibreStyle(base_style, icon_map, transient_scroller));
     return 0;
 }
 
