@@ -77,6 +77,6 @@ private:
     QColor m_dark, m_light;
 };
 
-int load_style(QHash<int,QString> icon_map);
+int load_style(QHash<int,QString> icon_map, int transient_scroller=0);
 void set_no_activate_on_click(QWidget *widget);
 void draw_snake_spinner(QPainter &painter, QRect rect, int angle, const QColor & light, const QColor & dark);
